	title	'LDRBIOS BIOS for CP/M 3.0'

	maclib	z80

	true	equ -1
	false	equ not true
	
	cr	equ 13
	lf	equ 10
	bell	equ 7
	ctlQ	equ 'Q'-'@'
	ctlS	equ 'S'-'@'

    ; variables in system data page

	extrn	fd$read, fd$write, fd$login, fd$init
	extrn	hd$read, hd$write, hd$login, hd$init
	extrn	fdsd0, fdsd1, fdsd2, fdsd3, fdsd4, fdsd5, fdsd6, fdsd7

    ; user defined character I/O routines

	extrn ?ci,?co,?cist,?cost	; each take device

    ; disk communication data items

	extrn @dtbl			; table of pointers to XDPHs
	public @adrv,@rdrv,@trk,@sect	; parameters for disk I/O
	public @dma,@dbnk,@cnt		;    ''       ''   ''  ''
	public @ermde			; BDOS error mode

    ; memory control

	public @cbnk			; current bank
	public ?bank

    ; general utility routines

	public ?pmsg,?pdec	; print message, print number from 0 to 65535
	public ?pderr		; print BIOS disk error message header

    ; External names for BIOS entry points

	public ?boot,?wboot,?const,?conin,?cono,?list,?auxo,?auxi
	public ?home,?sldsk,?sttrk,?stsec,?stdma,?read,?write
	public ?lists,?sctrn
	public ?conos,?auxis,?auxos,?dvtbl,?devin,?drtbl
	public ?mltio,?flush,?mov,?tim,?bnksl,?stbnk,?xmov


    ; BIOS Jump vector.

		; All BIOS routines are invoked by calling these
		;	entry points.

?boot:	jmp boot	; initial entry on cold start
?wboot:	jmp empty	; reentry on program exit, warm start

?const:	jmp empty	; return console input status
?conin:	jmp empty	; return console input character
?cono:	jmp conout	; send console output character
?list:	jmp empty	; send list output character
?auxo:	jmp empty	; send auxilliary output character
?auxi:	jmp empty	; return auxilliary input character

?home:	jmp home	; set disks to logical home
?sldsk:	jmp seldsk	; select disk drive, return disk parameter info
?sttrk:	jmp settrk	; set disk track
?stsec:	jmp setsec	; set disk sector
?stdma:	jmp setdma	; set disk I/O memory address
?read:	jmp read	; read physical block(s)
?write:	jmp empty	; write physical block(s)

?lists:	jmp empty	; return list device status
?sctrn:	jmp sectrn	; translate logical to physical sector

?conos:	jmp empty	; return console output status
?auxis:	jmp empty	; return aux input status
?auxos:	jmp empty	; return aux output status
?dvtbl:	jmp empty	; return address of device def table
?devin:	jmp empty	; change baud rate of device

?drtbl:	jmp empty	; return address of disk drive table
?mltio:	jmp empty	; set multiple record count for disk I/O
?flush:	jmp empty	; flush BIOS maintained disk caching

?mov:	jmp ?move	; block move memory to memory
?tim:	jmp empty	; Signal Time and Date operation
?bnksl:	jmp empty	; select bank for code execution and default DMA
?stbnk:	jmp empty	; select different bank for disk I/O DMA operations.
?xmov:	jmp empty	; set source and destination banks for one operation

	jmp empty	; reserved for future expansion
	jmp empty	; reserved for future expansion
	jmp empty	; reserved for future expansion


	; EMPTY, nothing to do
	;
empty:
?bank:
	ret

	
	; BOOT
	;	Initial entry point for system startup.

boot:
	mvi	a,0
	sta	@cbnk		; current bank

;	dcr	a
;	sta	@ermde		; BDOS error mode (0ffh)

	mvi	a,020h
	sta	9		; ?? haengt sich sonst nach
				; der CPMLDR Message auf und
				; bootet nicht weiter!
	ret


	; CONOUT
	;	Console Output.  Send character in <C>
	;			to all selected devices

conout:	
	mvi	b,0	; device
	call	?co
	ret


;	Disk I/O interface routines


	; SELDSK
	;	Select Disk Drive.  Drive code in <C>.
	;		Invoke login procedure for drive
	;		if this is first select.  Return
	;		address of disk parameter header
	;		in <HL>

seldsk:
	lxi	h,fdsd0	; fdsd0=A:, fdsd4=E:
	mvi	a,0	; 0=A: ... 4=E:
	sta	@adrv	; current drive
	ret


	; HOME
	;	Home selected drive.  Treated as SETTRK(0).

home:
	lxi b,0		; same as set track zero


	; SETTRK
	;	Set Track. Saves track address from <BC> 
	;		in @TRK for further operations.

settrk:
	mov l,c ! mov h,b
	shld @trk
	ret


	; SETSEC
	;	Set Sector.  Saves sector number from <BC>
	;		in @sect for further operations.

setsec:
	mov l,c ! mov h,b
	shld @sect
	ret


	; SETDMA
	;	Set Disk Memory Address.  Saves DMA address
	;		from <BC> in @DMA and sets @DBNK to @CBNK
	;		so that further disk operations take place
	;		in current bank.

setdma:
	mov l,c ! mov h,b
	shld @dma
	xra a		; Bank 0
	sta @dbnk
	ret


	; SECTRN
	;	Sector Translate.  Indexes skew table in <DE>
	;		with sector in <BC>.  Returns physical sector
	;		in <HL>.  If no skew table (<DE>=0) then
	;		returns physical=logical.

sectrn:
	mov l,c ! mov h,b
	mov a,d ! ora e ! rz
	xchg ! dad b ! mov l,m ! mvi h,0
	ret


	; READ
	;	Read physical record from currently selected drive.
	;		Finds address of proper read routine from
	;		extended disk parameter header (XDPH).

read:
	lhld @adrv ! mvi h,0 ! dad h	; get drive code and double it
	lxi d,@dtbl ! dad d		; make address of table entry
	mov a,m ! inx h ! mov h,m ! mov l,a	; fetch table entry
	push h				; save address of table
	lxi d,-8 ! dad d		; point to read routine address
rw$common:
	mov a,m ! inx h ! mov h,m ! mov l,a	; get address of routine
	pop d				; recover address of table
	dcx d ! dcx d			; point to relative drive
	ldax d ! sta @rdrv		; get relative drive code and post it
	inx d ! inx d			; point to DPH again
	pchl				; leap to driver

	; ?MOVE
	;
?move:
	xchg
	ldir
	xchg
	ret

	; error message components

drive$msg	db	cr,lf,bell,'BIOS Error on ',0
track$msg	db	': T-',0
sector$msg	db	', S-',0


    ; disk communication data items

@adrv	ds	1		; currently selected disk drive
@rdrv	ds	1		; controller relative disk drive
@trk	ds	2		; current track number
@sect	ds	2		; current sector number
@dma	ds	2		; current DMA address
@cnt	db	0		; record count for multisector transfer
@dbnk	db	0		; bank for DMA operations

@cbnk	db	0		; current bank
@ermde	db	00ffh		; BDOS error mode		, 

;	Utility Subroutines

?pmsg:		; print message @<HL> up to a null
		; saves <BC> & <DE>
	push b
	push d
pmsg$loop:
	mov a,m ! ora a ! jz pmsg$exit
	mov c,a ! push h
	call ?cono ! pop h
	inx h ! jmp pmsg$loop
pmsg$exit:
	pop d
	pop b
	ret

?pdec:		; print binary number 0-65535 from <HL>
	lxi b,table10! lxi d,-10000
next:
	mvi a,'0'-1
pdecl:
	push h! inr a! dad d! jnc stoploop
	inx sp! inx sp! jmp pdecl
stoploop:
	push d! push b
	mov c,a! call ?cono
	pop b! pop d
nextdigit:
	pop h
	ldax b! mov e,a! inx b
	ldax b! mov d,a! inx b
	mov a,e! ora d! jnz next
	ret

table10:
	dw	-1000,-100,-10,-1,0

?pderr:
	lxi h,drive$msg ! call ?pmsg			; error header
	lda @adrv ! adi 'A' ! mov c,a ! call ?cono	; drive code
	lxi h,track$msg ! call ?pmsg			; track header
	lhld @trk ! call ?pdec				; track number
	lxi h,sector$msg ! call ?pmsg			; sector header
	lhld @sect ! call ?pdec				; sector number
	ret

	end
